import java.awt.*;

class ImageCanvas extends Canvas {
   final static int STATE_FRONT=0;
   final static int STATE_BACK=1;
   final static int INVALID_ID=-1;

   Color bgColor=null;

   Image image=null;
   int imageCanvasId=INVALID_ID;
   int imageId=INVALID_ID;
   int state=STATE_FRONT;
   boolean selected=false;

   public ImageCanvas(int imageCanvasId) {
      this.imageCanvasId = imageCanvasId;
      this.imageId = INVALID_ID;
      this.image = null;
      init();
   }

   public ImageCanvas(int imageCanvasId, int imageId, Image image) {
      this.imageCanvasId = imageCanvasId;
      this.imageId = imageId;
      this.image = image;
      init();
   }

   public void init() {
      state = STATE_FRONT;
      if(image == null) {
         setSize(0, 0);
      }
      bgColor = new Color(0xef, 0xf8, 0xff);
   }

   public boolean equals(Object o) {
      if((o != null)&&(o instanceof ImageCanvas)) {
         ImageCanvas p = (ImageCanvas)o;
         if(p.getId() == this.imageId) {
            return(true);
         }
      }
      return(false);
   }

   public int getState() {
      return(state);
   }

   public int getId() {
      return(imageId);
   }

   public boolean isSelected() {
      return(selected == true);
   }

   public void select() {
      if(selected) {
         return;
      }
      selected = true;
      repaint();
   }

   public boolean isDeselected() {
      return(selected == false);
   }

   public void deselect() {
      if(!selected) {
         return;
      }
      selected = false;
      repaint();
   }

   public void setId(int imageId) {
      this.imageId = imageId;
   }

   public int getImageCanvasId() {
      return(imageCanvasId);
   }

   public void setImageCanvasId(int imageCanvasId) {
      this.imageCanvasId = imageCanvasId;
   }

   public void reverseState() {
      if(state == STATE_FRONT) {
         state = STATE_BACK;
      } else {
         state = STATE_FRONT;
      }
/*
      System.out.println("State: "+state);
*/
   }

   public void setState(int state) {
      this.state = state;
   }

   public Image getImage() {
      return(image);
   }

   public void setImage(Image image) {
      this.image = image;
      init();
   }

   public int getWidth() {
      return(getSize().width);
   }

   public int getHeight() {
      return(getSize().height);
   }

   public void repaint() {
      paint(getGraphics());
   }

   public void update(Graphics g) {
      paint(g);
   }

   public void paint(Graphics g) {
      if((image != null)&&(state == STATE_FRONT)) {
         g.drawImage(image, 0, 0, getWidth(), getHeight(), this);
         g.setColor(Color.black);
         g.drawRect(0, 0, getWidth()-1, getHeight()-1);
      } else {
         g.setColor(bgColor);
         g.fillRect(0, 0, getWidth()-1, getHeight()-1);
         g.setColor(Color.black);
         g.drawRect(0, 0, getWidth()-1, getHeight()-1);
      }
      if(isSelected()) {
         g.setColor(Color.blue);
         g.drawRect(0, 0, getWidth()-1, getHeight()-1);
         g.drawRect(1, 1, getWidth()-3, getHeight()-3);
         g.drawRect(2, 2, getWidth()-5, getHeight()-5);
      }
   }
}

